@@ -135,9 +135,12 @@ class Client(object):
         """
 
         if type(data) is list:
-            for record in data:
+            for index, record in enumerate(data):
                 for key, value in record.items():
-                    record.update({key: {'value': value}})
+                    if value != None:
+                        record.update({key: {'value': value}})
+                    else:
+                        print(f'Removing {value} from record item {index} because it is null (Nonetype)')
 
         return data
 
@@ -149,7 +152,7 @@ class Client(object):
 
         params = {
             'tableId': database,
-            'includeFieldPerms': include_permissions
+            'includeFieldPerms': 'true' if include_permissions else 'false'
         }
 
         tries = 0
@@ -173,7 +176,13 @@ class Client(object):
 
                 pass
 
-        return response.json()
+        response = response.json()
+        if type(response) == dict:
+            if response.get('message') == 'Bad Request':
+                desc = response.get('description')
+                raise ResponseError(code=-7, msg=desc)
+
+        return response
 
     def get_primary_key(self, database=None):
         """Returns the primary key of a given table"""
@@ -211,13 +220,15 @@ class Client(object):
         else:
             return []
 
-    def do_query(self, query=None, qid=None, columns=None, sort=[3],
+    def do_query(self, query=None, qid=None, columns=None, sort=None,
                  num=None, skip=None, ascending=True,
                  database=None, round_ints=False, require_all=False):
         """Perform query and return results (list of dicts)."""
 
         if len([q for q in (query, qid) if q]) != 1:
             raise TypeError('must specify one of query, qid')
+        if sort == None:
+            sort = [3]
         # regular query
         if query:
             try:
@@ -267,6 +278,10 @@ class Client(object):
                     params=params,
                     headers=self.headers)
                 response = response.json()
+                if type(response) == dict:
+                    if response.get('message') == 'Bad Request':
+                        desc = response.get('description')
+                        raise ResponseError(code=-7, msg=desc)
 
                 fields = columns
                 where = response['query']['filter']
@@ -376,8 +391,13 @@ class Client(object):
             'skip': skip,
             'top': top
         }, headers=self.headers)
+        response = response.json()
+        if type(response) == dict:
+            if response.get('message') == 'Bad Request':
+                desc = response.get('description')
+                raise ResponseError(code=-7, msg=desc)
 
-        return response.json()
+        return response
 
     def perform_query(self, body=None):
 
@@ -387,7 +407,13 @@ class Client(object):
             json=body
         )
 
-        return response.json()
+        response = response.json()
+        if type(response) == dict:
+            if response.get('message') == 'Bad Request':
+                desc = response.get('description')
+                raise ResponseError(code=-7, msg=desc)
+
+        return response
 
     def add_record(self, fields, database=None, uploads=None, safemode=False):
         """
@@ -415,7 +441,8 @@ class Client(object):
 
         # add fields and field values to record
         for field, value in fields.items():
-            record.update({field: {"value": value}})
+            if value != None:
+                record.update({field: {"value": value}})
         # add uploads to record
         if uploads:
             for upload in uploads:
@@ -428,6 +455,10 @@ class Client(object):
 
         # convert the response to a json object
         response = response.json()
+        if type(response) == dict:
+            if response.get('message') == 'Bad Request':
+                desc = response.get('description')
+                raise ResponseError(code=-7, msg=desc)
 
         try:
 
@@ -520,7 +551,7 @@ class Client(object):
 
         records = ""
         if rids:
-            "OR".join(["{3.EX.%s}" % rid for rid in rids])
+            records = "OR".join(["{3.EX.%s}" % rid for rid in rids])
         elif query:
             records = query
         else:
@@ -534,9 +565,13 @@ class Client(object):
 
         response = requests.delete(url, headers=headers, json=body)
 
-        res_json = response.json()
+        response = response.json()
+        if type(response) == dict:
+            if response.get('message') == 'Bad Request':
+                desc = response.get('description')
+                raise ResponseError(code=-7, msg=desc)
 
-        return res_json.get("numberDeleted")
+        return response.get("numberDeleted")
 
     def get_file(self, url=None, database=None, record=None, field=None, version=1):
         """
@@ -629,6 +664,9 @@ class Client(object):
         if type(first_field) != dict or (type(first_field) == dict and 'value' not in first_field):
             data = self.nest(data)
 
+        if return_fields == None:
+            return_fields = [3]
+
         # if safemode is true, check if keyfield has been passed in as parameter
         if safemode:
             primarykey = self.get_primary_key(
@@ -652,6 +690,10 @@ class Client(object):
 
         response = requests.post(url=url, headers=headers, json=body)
         response = response.json()
+        if type(response) == dict:
+            if response.get('message') == 'Bad Request':
+                desc = response.get('description')
+                raise ResponseError(code=-7, msg=desc)
 
         if round_ints:
             response['data'] = self.round_ints(response['data'])
@@ -685,6 +727,9 @@ class Client(object):
         primary_key = self.get_primary_key(
             database=database or self.database)
 
+        if return_fields == None:
+            return_fields = [3]
+
         # return an error if any record is missing its primary key
         for record in data:
             if str(primary_key) not in record:
@@ -710,6 +755,10 @@ class Client(object):
 
         response = requests.post(url=url, headers=headers, json=body)
         response = response.json()
+        if type(response) == dict:
+            if response.get('message') == 'Bad Request':
+                desc = response.get('description')
+                raise ResponseError(code=-7, msg=desc)
 
         if round_ints:
             response['data'] = self.round_ints(response['data'])
@@ -755,7 +804,8 @@ class Client(object):
 
             # add fields and field values to record
             for field, value in fields.items():
-                record.update({field: {"value": value}})
+                if value != None:
+                    record.update({field: {"value": value}})
             # add uploads to record
             if uploads:
                 for upload in uploads:
@@ -768,6 +818,10 @@ class Client(object):
 
             # convert the response to a json object
             response = response.json()
+            if type(response) == dict:
+                if response.get('message') == 'Bad Request':
+                    desc = response.get('description')
+                    raise ResponseError(code=-7, msg=desc)
 
             try:
 
